@@ -105,6 +105,18 @@ _priv_gst_caps_features_initialize (void)
       &static_caps_features_parent_refcount);
 }
 
+void
+_priv_gst_caps_features_cleanup (void)
+{
+  gst_caps_features_set_parent_refcount (_gst_caps_features_any, NULL);
+  gst_caps_features_free (_gst_caps_features_any);
+  _gst_caps_features_any = NULL;
+  gst_caps_features_set_parent_refcount
+      (_gst_caps_features_memory_system_memory, NULL);
+  gst_caps_features_free (_gst_caps_features_memory_system_memory);
+  _gst_caps_features_memory_system_memory = NULL;
+}
+
 gboolean
 gst_is_caps_features (gconstpointer obj)
 {
